/*
 * lab1.c
 */

#include "lcd.h"
#include "usart.h"

#include <avr/io.h>
#include <util/delay.h>


#define MORSE_LUNG  3
#define MORSE_SCURT 1

char * morse[] = {
    "13",
    "3111",
    "3131",
    "311",
    "1",
    "1131",
    "331",
    "1111",
    "11",
    "1333",
    "313",
    "1311",
    "33",
    "31",
    "333",
    "1331",
    "3313",
    "131",
    "111",
    "111",
    "3",
    "113",
    "1113",
    "133",
    "3113",
    "3133",
    "3311"};



void exemplu_usart()
{
    for(;;)
    {
        USART0_print("Salut!");
        _delay_ms(10);
    }
}

void task1_usart()
{
    /* TODO task 1 USART */

    // configurare porturi folosite
    DDRB &= ~((1 << PB0) | (1 << PB1));
    PORTB |= (1 << PB0) | (1 << PB1);

    // main loop
    int8_t PB0_flag = 0, PB1_flag = 0;
    for(;;)
    {
        // cand PB0 este apasat trimite un mesaj pe USART
        if(!(PINB & (1 << PB0)) && !PB0_flag)
        {
            PB0_flag = 1;
            USART0_print("PB0 apasat\n");
        }

        // cand PB0 este lasat trimite un mesaj pe USART
        if((PINB & (1 << PB0)) && PB0_flag)
        {
            PB0_flag = 0;
            USART0_print("PB0 lasat\n");
        }

        // cand PB1 este apasat trimite un mesaj pe USART
        if(!(PINB & (1 << PB1)) && !PB1_flag)
        {
            PB1_flag = 1;
            USART0_print("PB1 apasat\n");
        }

        // cand PB1 este lasat trimite un mesaj pe USART
        if((PINB & (1 << PB1)) && PB1_flag)
        {
            PB1_flag = 0;
            USART0_print((char *)"PB1 lasat\n");
        }

        // much switch, so debouncing, such delay
        _delay_ms(5);
    }
}

void task2_usart()
{
    /* TODO task 2 USART */

    // configurari porturi folosite
    DDRB &= ~((1 << PB0) | (1 << PB1));
    PORTB |= (1 << PB0) | (1 << PB1);

    // configurarea stdout
    FILE mystdout;
    mystdout.put = USART0_printf;
    mystdout.flags = __SWR;
    stdout = &mystdout;

    // main loop
    int8_t PB0_flag = 0, PB1_flag = 0;
    for(;;)
    {
        // cand PB0 este apasat trimite un mesaj pe USART
        if(!(PINB & (1 << PB0)) && !PB0_flag)
        {
            PB0_flag = 1;
            printf("PB0 apasat\n");
        }

        // cand PB0 este lasat trimite un mesaj pe USART
        if((PINB & (1 << PB0)) && PB0_flag)
        {
            PB0_flag = 0;
            printf("PB0 lasat\n");
        }

        // cand PB1 este apasat trimite un mesaj pe USART
        if(!(PINB & (1 << PB1)) && !PB1_flag)
        {
            PB1_flag = 1;
            printf("PB1 apasat\n");
        }

        // cand PB1 este lasat trimite un mesaj pe USART
        if((PINB & (1 << PB1)) && PB1_flag)
        {
            PB1_flag = 0;
            printf("PB1 lasat\n");
        }

        // much switch, so debouncing, such delay
        _delay_ms(5);
    }
}

/* apelati functia pentru a obtine un ton lung/scurt,
 * folosind macro-urile MORSE_LUNG si MORSE_SCURT
 */
void speaker_morse(int tip)
{
    int delay = 1;
    int i;
    DDRD |= (1 << PD5);     // PD5 trebuie sa fie output

    for (i = 0; i < tip * 50 / delay; i++)
    {
        PORTD ^= (1 << PD5); // alternam starea iesirii
        _delay_ms(1);
    }
}

void task3_usart()
{
    /* TODO task 3 USART */

    // main loop
    char c;
    for(;;)
    {
        c = USART0_receive();

        if (c >= 'a' && c <= 'z')
        {
            UDR0 = c;
            char * m = morse[c - 'a'];
            while (*m)
            {
                speaker_morse(*m - '0');
                _delay_ms(50);
                m++;
            }
            _delay_ms(50);
        }
    }
}


void exemplu_lcd()
{
    LCD_print("Hello World!");

    for(;;);
}

void task1_lcd()
{
    /* TODO task 1 LCD */

    // configurare porturi folosite
    DDRB &= ~((1 << PB0) | (1 << PB1));
    PORTB |= (1 << PB0) | (1 << PB1);

    // initializari
    char str[] = "Hello World";
    LCD_print(str);

    // main loop
    int8_t pos = 0;
    uint8_t PINB_old = 0xff;
    for(;;)
    {
        uint8_t PINB_crt = PINB;

        if((PINB_old & (1 << PB1)) && !(PINB_crt & (1 << PB1)))
        {
            pos += 1;
            if(pos + (sizeof(str) - 1) > 16)
                pos = 16 - (sizeof(str) - 1);

            LCD_writeInstr(LCD_INSTR_clearDisplay);
            LCD_printAt(pos, str);
        }

        if((PINB_old & (1 << PB0)) && !(PINB_crt & (1 << PB0)))
        {
            pos -= 1;
            if(pos < 0)
                pos = 0;

            LCD_writeInstr(LCD_INSTR_clearDisplay);
            LCD_printAt(pos, str);
        }

        PINB_old = PINB_crt;

        _delay_ms(5);
    }
}

void task2_lcd()
{
    /* TODO task 2 LCD */

    // main loop
    uint8_t pos = 0;
    for(;;)
    {
        LCD_putCharAt(pos++, USART0_receive());

        switch(pos)
        {
            case 0x10:
                pos = 0x40;
                break;
            case 0x50:
                pos = 0x00;
                break;
        }
    }
}

void bonus_lcd()
{
    /* TODO bonus LCD */

    // configurare porturi folosite
    DDRB &= ~((1 << PB0) | (1 << PB1));
    PORTB |= (1 << PB0) | (1 << PB1);

    // creare caracter custom
    uint8_t chuck[] = {0x00, 0x0e, 0x0e, 0x04, 0x0e, 0x04, 0x0e, 0xa};
    LCD_writeInstr(LCD_INSTR_CGRAM);
    for(uint8_t i = 0; i < sizeof(chuck); ++i)
        LCD_writeData(chuck[i]);
    LCD_writeInstr(LCD_INSTR_DDRAM);

    // main loop
    int8_t pos_old = 0x40;
    uint8_t PINB_old = 0xff;
    LCD_putCharAt(pos_old, 0);
    for(;;)
    {
        uint8_t PINB_crt = PINB;

        uint8_t pb0 = (PINB_old & (1 << PB0)) && !(PINB_crt & (1 << PB0));
        uint8_t pb1 = (PINB_old & (1 << PB1)) && !(PINB_crt & (1 << PB1));

        int8_t pos = pos_old;

        if(!(PINB_crt & (1 << PB0)) && !(PINB_crt & (1 << PB1)))
        {
            if(pos >= 0x40)
                pos -= 0x40;
        }
        else if(pb0)
        {
            pos -= 1;
            if(pos == 0x3f)
                pos = 0x40;
        }
        else if(pb1)
        {
            pos += 1;
            if(pos == 0x50)
                pos = 0x4f;
        }
        else
        {
            if(pos <= 0x1f)
                pos += 0x40;
        }

        if(pos != pos_old)
        {
            LCD_writeInstr(LCD_INSTR_clearDisplay);
            LCD_putCharAt(pos, 0);

            pos_old = pos;
        }

        PINB_old = PINB_crt;

        _delay_ms(5);
    }
}

int main()
{
    //USART
    USART0_init();

  exemplu_usart();
//  task1_usart();
//  task2_usart();
//    task3_usart();

    //LCD
    LCD_init();

//  exemplu_lcd();
//  task1_lcd();
//  task2_lcd();
//  bonus_lcd();

    return 0;
}

